function DropDownMenu()
{
    this.oDropdown = null;
    this.savedSmartListID = 0;
    this.typekitEnabled = false;
    this.googleFontsEnabled = false;
    this.digests = null;
    this.deleteButton = null;
    this.createButton = null;
    this.editButton = null;

    this.getTypeKitEnabled = function()
    {
        return this.typekitEnabled;
    };

    this.setTypeKitEnabled = function(enabled)
    {
        if(enabled != this.typekitEnabled)
        {
            this.typekitEnabled = enabled;
            this.populateDropDownMenu();
        }
    };

    this.getGoogleFontsEnabled = function()
    {
        return this.googleFontsEnabled;
    };

    this.setGoogleFontsEnabled = function(enabled)
    {
        if(enabled != this.googleFontsEnabled)
        {
            this.googleFontsEnabled = enabled;
            this.populateDropDownMenu();
        }
    };
    
    this.areGoogleFontsEnabledHandler = function(isEnabled, isReplicating)
    {
        this.setGoogleFontsEnabled(isEnabled);
    };
    
    this.typeKitEnabledHandler = function(isEnabled)
    {
        this.setTypeKitEnabled(isEnabled);
    };
    
    this.getSmartListName = function(smartListID)
    {
        var result = null;

        for(var i = 0; i < this.oDropdown.options.length; ++i)
        {
            if(parseInt(this.oDropdown.options[i].value) == smartListID)
            {
                result = this.oDropdown.options[i].text;
                break;
            }
        }
        
        return result;
    };

    this.getSelectedSmartListName = function()
    {
        return this.getSmartListName(this.savedSmartListID);
    };

    this.setSmartListName = function(smartListID, name)
    {
        for(var i = 0; i < this.oDropdown.options.length; ++i)
        {
            if(parseInt(this.oDropdown.options[i].value) == smartListID)
            {
                this.oDropdown.options[i].innerText = name;
            }
        }
    };

    this.setSelectedItemByID = function(smartListID)
    {
        var oldSmartListID = this.savedSmartListID;
        var selectedIndex  = 0;

        // find the menu item index 
        for(var i = 0; i < this.oDropdown.options.length; ++i)
        {
            if(parseInt(this.oDropdown.options[i].value) == smartListID)
            {
                selectedIndex = this.oDropdown.options[i].index;
            }
        }
        
        this.oDropdown.set("selectedIndex", selectedIndex);
        
        // reconcile the correct smart list ID in the case where
        // we tried to select one that doesn't exist in the menu
        var actualSmartListID;
        
        if(selectedIndex < this.oDropdown.options.length)
        {
            actualSmartListID = parseInt(this.oDropdown.options[selectedIndex].value);
        }
        else
        {
            actualSmartListID = SmartListIDs.kDesktopFonts;
        }
 
        // if the smart list has changed, then reload
        if(oldSmartListID != actualSmartListID)
        {
            panel.fontList.clearSearch();
            panel.setSelectedRow(null);
            panel.messageSender.getSmartListFonts(actualSmartListID);
            this.savedSmartListID = actualSmartListID;
            panel.messageSender.saveDropDownSelection(actualSmartListID);
            this.updateButtons(actualSmartListID);
        }
    };

    this.updateButtons = function(smartListID)
    {
        var isEditable = this.isUserSmartList(smartListID);
        
        var buttons = [ "edit-smart-list-button", "delete-smart-list-button" ];
        
        if (isEditable && panel.isOnline)
        {
            for (var buttonNum = 0; buttonNum < buttons.length; ++buttonNum)
            {
                panel.enableSmartListButton(buttons[buttonNum])
            }
        }
        else
        {
            for (var buttonNum = 0; buttonNum < buttons.length; ++buttonNum)
            {
                panel.disableSmartListButton(buttons[buttonNum])
            }
        }
    };
    
    this.onMenuClick = function(e)
    {
        //"this" is really the msdropdown20 class and not DropDownMenu
        if (e.currentTarget.value != "")
        {
            panel.dropDownMenu.setSelectedItemByID(parseInt(e.currentTarget.value));
            panel.dropDownMenu.savedSmartListID = parseInt(e.currentTarget.value);
            
            switch(parseInt(panel.dropDownMenu.savedSmartListID))
            {
                case SmartListIDs.kGoogle:
                    if(panel.progress.googleProgressInfo.progressRunning)
                    {
                        panel.progress.updateProgress(ProgressTitle.kSynchronizeGoogle, panel.progress.googleProgressInfo);
                    }
                    else
                    {
                        panel.progress.hideProgress();
                    }
                    break;
                    
                case SmartListIDs.kTypekit:
                case SmartListIDs.kSystemFonts:
                    panel.progress.hideProgress();
                    break;
                    
                default:
                    if(panel.progress.bulkReplicationProgressInfo.progressRunning)
                    {
                        panel.progress.updateProgress(ProgressTitle.kSynchronizeUTS, panel.progress.bulkReplicationProgressInfo);
                    }
                    else
                    {
                        panel.progress.hideProgress();
                    }
                    break;
            }
        }
        else
        {
            panel.dropDownMenu.setSelectedItemByID(this.savedSmartListID);
        }
    };
    
    this.populateDropDownMenu = function()
    {
        if(this.oDropdown != null)
        {
            if(this.oDropdown.options.length > 0)
            {
                for(var i = this.oDropdown.options.length - 1; i >= 0; --i)
                {
                    this.oDropdown.remove(this.oDropdown.options[i].index);
                }
            }
            
            var icon = DropdownIcon.kMyFontsUTC;
            if ( panel.isRunningStandalone() )
            {
                icon = DropdownIcon.kMyFontsStandalone;
            }
            this.oDropdown.add({value: SmartListIDs.kDesktopFonts, text:DropdownTitle.kMyFontsWorkgroup, image:icon});
            this.oDropdown.add({value: SmartListIDs.kSystemFonts, text:DropdownTitle.kSystemFontsWorkgroup, image:DropdownIcon.kSystemFonts});
            if ( this.getGoogleFontsEnabled() || this.getTypeKitEnabled() )
            {
                this.oDropdown.add({className:"disabled", value:'Separator', text:'', disabled:true});
                
                if ( this.getTypeKitEnabled() )
                {
                    this.oDropdown.add({value: SmartListIDs.kTypekit, text:DropdownTitle.kTypekitWorkgroup, image:DropdownIcon.kTypekit});            
                }
                if ( this.getGoogleFontsEnabled() )
                {
                    this.oDropdown.add({value: SmartListIDs.kGoogle, text:DropdownTitle.kGoogleWorkgroup, image:DropdownIcon.kGoogle});            
                }
            }
        
            if ( this.digests && this.digests.length )
            {
                this.oDropdown.add({value:'Separator', text:'', disabled:true});
                
                var icon = this.getCustomSmartListIcon();

                for (var index in this.digests) 
                {
                    this.oDropdown.add({value:this.digests[index][MessageKey.kSmartListIDKey], text:this.digests[index][MessageKey.kSmartListNameKey], image:icon});
                }
            }
    
            panel.messageSender.getDropDownSelection();
            resizeElements();
        }
    };
    
    this.getCustomSmartListIcon = function()
    {
        var icon = null;
        if ( panel.isRunningStandalone() )
        {
            icon = DropdownIcon.kSmartListStandalone;
        }
        else
        {
            icon = DropdownIcon.kSmartListUTC;
        }
        
        return icon;
    }

    this.smartListCreatedHandler = function(smartListID, smartListName)
    {
        // temporarily add the new smart list to the drop down menu
        // so that we can edit it.  In the meantime, downstream
        // activities will force a reload
        this.oDropdown.add({value:smartListID, text:smartListName, image:this.getCustomSmartListIcon()});
        this.setSelectedItemByID(smartListID);

        panel.messageSender.saveDropDownSelection(smartListID);
        panel.messageSender.getAllSmartLists();
    };
    
    this.onDeleteButtonClick = function() 
    {
        panel.fontList.clearSearch();

        var smartListID = this.savedSmartListID;
        
        var message = localizeString("AreYouSureDeleteSmartList");
        message = message.replace("{0}", this.getSmartListName(smartListID));
        
        panel.showAlert("",
            message,
            localizeString("Yes"),
            function()
            {
                panel.messageSender.deleteSmartList(smartListID);        
                panel.messageSender.saveDropDownSelection(SmartListIDs.kDesktopFonts);
                panel.messageSender.getAllSmartLists();  
            },
            localizeString("No"),
            null
        );
    };
    
    this.onCreateButtonClick = function()
    {
        panel.fontList.clearSearch();

        // Rely on handleMessage to reload and then select the new smartlist.
        panel.messageSender.createSmartList();
    };
    
    this.onEditButtonClick = function()
    {
        panel.fontList.clearSearch();

        panel.messageSender.getSmartListTree(this.savedSmartListID);
    };
    
    this.initializeDropDownMenu = function()
    {
        var jsonData = [];
        this.oDropdown = $(".drop-down-menu").msDropdown({mainCSS:'dropMenu',
                                                          visibleRows:12,
                                                          byJson:{data:jsonData, name:'font-digest'}}).data("dd");
        
        this.populateDropDownMenu();
        
        this.oDropdown.on("change", this.onMenuClick);

        this.deleteButton = document.getElementById("delete-smart-list-button");
        this.createButton = document.getElementById("create-smart-list-button");
        this.editButton = document.getElementById("edit-smart-list-button");

        this.deleteButton.onclick = function() {
            panel.dropDownMenu.onDeleteButtonClick();
        };
        this.createButton.onclick = function() {
            panel.dropDownMenu.onCreateButtonClick();
        };
        this.editButton.onclick = function() {
            panel.dropDownMenu.onEditButtonClick();
        };
        
        resizeElements();
    };

    this.dropDownMenuHandler = function(digests)
    {
        this.digests = digests;
        this.initializeDropDownMenu();
    };

    this.dropDownSelectionHandler = function(selection)
    {
        this.setSelectedItemByID(selection);
    };
    
    this.isCannedSmartList = function(smartListID)
    {
        return smartListID >= 0xffffff00;
    }
    
    this.isUserSmartList = function(smartListID)
    {
        return !this.isCannedSmartList(smartListID);
    }
    
    this.currentSmartListIsCannedSmartList = function()
    {
        return this.isCannedSmartList(this.savedSmartListID);
    }
    
    this.currentSmartListIsUserSmartList = function()
    {
        return !this.currentSmartListIsCannedSmartList();
    }
    
    this.currentSmartListIsReloadable = function()
    {
        return this.currentSmartListIsUserSmartList() ||
            this.savedSmartListID == SmartListIDs.kDesktopFonts;
    }
};


