var loadingScreenSpinner = new IndeterminateSpinner("initial-loading-image");

function onLoaded()
{
    var csInterface = new CSInterface();

    updateThemeWithAppSkinInfo(csInterface.hostEnvironment.appSkinInfo);
    // Update the color of the panel when the theme color of the product
    // changed.
    csInterface.addEventListener(CSInterface.THEME_COLOR_CHANGED_EVENT,
            onAppThemeColorChanged);
    
    document.addEventListener("contextmenu", function(e)
    {
        e.preventDefault();
    }, false);
    
    if (debugPanel)
    {
        var button = document.createElement("button");
        var text = document.createTextNode("Start");
        button.appendChild(text);
        
        button.onclick = function()
        {
            preLaunchSetup();
        };
        
        document.body.insertBefore(button, document.body.children[0]);
        hideInitialLoadingScreen();
        $("#connection-failed-screen").hide();
    }
    else
    {
        preLaunchSetup(true);
    }
};

function onKeyDown(event)
{
    // If an F key or the ctrl key plus a non modifier key is pressed
    if((event.keyCode >= Keyboard.F1 && event.keyCode <= Keyboard.F12) || 
       (event.ctrlKey && 
        event.keyCode != Keyboard.CTRL && event.keyCode != Keyboard.SHIFT && event.keyCode != Keyboard.ALT)) // The control key and another key that isn't the control, shift, or alt key
    {
        panel.messageSender.sendHostAppKeyPress(event.ctrlKey, event.shiftKey, event.altKey, event.keyCode);
        return false;
    }
    
    return true;
};


function hideInitialLoadingScreen()
{
    $("#initial-loading-screen").hide();
    loadingScreenSpinner.startSpinner();
};

function preLaunchSetup(firstTry)
{
    loadingScreenSpinner.startSpinner();
    
    //requestFirstLaunchInfo will call startPanel()
    //via panel.handleFirstLaunchInfo()
    
    panel.startupTimer = window.setTimeout(function()
        {   
            if (firstTry == true) 
            {
                preLaunchSetup(false)
            }
            else
            {
                hideInitialLoadingScreen(); // Shows the error message behind it
            }
        }, 15000);
    
    localizeHTML();
    
    registerHandlers();
    panel.messageSender.requestStartPanelInfo();        
};

function PSCallback(csEvent)
{
    console.log(csEvent);
}

function startPanel()
{
    window.clearTimeout(panel.startupTimer);
    panel.startupTimer = null;

    /*
     * In Suitcase, we're ready to go.  In UTC, we need to
     * see whether we're actually logged in.
     */
    if(panel.isRunningStandalone())
    {
        hideInitialLoadingScreen();
        $("#connection-failed-screen").hide();
    }
    else
    {
        // Request logged in status, returning us to handleUTSIsLoggedIn()
        panel.messageSender.getUTSIsLoggedIn();
    }
    
    panel.messageSender.areGoogleFontsEnabled();
    panel.messageSender.getAllSmartLists();
    
    if(panel.isPhotoshop())
    {
        loadJSX("Photoshop.jsx");
        
        var csInterface = new CSInterface();

        // Panel JavasSript doesn't have a good wayt o call charIDToTypeID,
        // so we just store the precomputed values
        // 1936483188 = charIDToTypeID("slct")
        // 1936028772 = charIDToTypeID("setd")

        var eventSelect = 1936483188;
        var eventSet    = 1936028772;

        csInterface.addEventListener("PhotoshopCallback",
            function(csEvent)
            {
                csInterface.evalScript('updateSelection()',
                    function(psName)
                    {
                        if (!panel.fontList.isApplyingFont())
                        {
                            panel.fontList.selectRowByPostscriptName(psName);
                        }
                    }
                );
            }
        );

        var event = new CSEvent("com.adobe.PhotoshopRegisterEvent", "APPLICATION");
        event.extensionId = "com.extensis.HTMLPanel.Photoshop";
        event.data = eventSelect + ", " + eventSet;

        csInterface.dispatchEvent(event);
    }
    else if(panel.isIllustrator())
    {
        loadJSX("Illustrator.jsx");
    }
    else if(panel.isInDesign())
    {
        loadJSX("InDesign.jsx");
    }
    

    if (panel.isWindows())
    {
        $("<link/>", {
            rel: "stylesheet",
            type: "text/css",
            href: "./filters.css"
         }).appendTo("head");
    }
    
    $("html").click(function()
        {
            $(".drop-down-menu").msDropDown().data("dd").close();
            
            var tooltipObject = panel.getTooltipObject();
            if(tooltipObject)
            {
                tooltipObject.removeChild(tooltipObject.tooltip);
                panel.setTooltipObject(null);
            }
        });

    // defer resizing, otherwise it doesn't work
    window.setTimeout(function() { resizeElements(); }, 10);
    
    $(document).mousemove(function(e) 
    {
        panel.setMouseX(e.pageX);
        panel.setMouseY(e.pageY);
    });
    
    var buttons = ["edit-smart-list-button",
                   "create-smart-list-button",
                   "delete-smart-list-button"];
    
    for (var buttonNum = 0; buttonNum < buttons.length; ++buttonNum)
    {
        var button = document.getElementById(buttons[buttonNum]);
        
        $(button)
        .mouseenter(function()
        {
            if (this.tooltip != null)
            {
                panel.setTooltipObject(this);

                this.appendChild(this.tooltip);
                
                var hoverButton = this;
                
                this.hoverTimer = setTimeout(function()
                {
                    $(hoverButton.tooltip).css("opacity", 1);
                    
                    var y = panel.getMouseY() + 20;
                    var x = panel.getMouseX() + 15;
                                    
                    if(x + hoverButton.tooltip.clientWidth > document.body.clientWidth)
                    {
                        x = document.body.clientWidth - hoverButton.tooltip.clientWidth;
                    }
                    
                    $(hoverButton.tooltip).css("top", y);
                    $(hoverButton.tooltip).css("left", x);
                }, 1000);
            } 
        })
        .mouseleave(function() 
        {
            if(this.hoverTimer != null)
            {
                clearTimeout(this.hoverTimer);
                
                $(this.tooltip).css("opacity", 0);
                
                if (this.tooltip && this.tooltip.parentNode === this)
                {  
                    this.removeChild(this.tooltip);
                } 
                panel.setTooltipObject(null);
            }
        });
    }
    
    $(".button-mouse-area").css("height", $(".drop-menu-buttons").css("height"));
        
    $(".button-mouse-area").mouseenter(function()
    {
        $(this).next("button").mouseenter();
    });
    
    $(".button-mouse-area").mouseleave(function()
    {
        $(this).next("button").mouseleave();
    });
}


function handleUTSIsLoggedIn(loggedIn, isOnline)
{
    hideInitialLoadingScreen();
    $("#connection-failed-screen").hide();
    
    panel.setUTSIsLoggedIn(loggedIn, isOnline);
}

function resizeElements()
{
    var windowHeight = window.innerHeight;
    var windowWidth = window.innerWidth;

    var windowHeightPx = windowHeight + "px";
    var windowWidthPx = windowWidth + "px";

    var fontList = document.getElementById("font-list");

    /*
     * Adjust the drop-down menu's width
     */
    var fontListWidth = $(fontList).width();
    var dropDownMenu = document.getElementById("msdropdown20_msdd");
    var desiredWidth = fontListWidth - $('#dropdown-right').width();
    if (desiredWidth != $(dropDownMenu).width())
    {
        $(dropDownMenu).width(desiredWidth);
    }
    
    const dropDownMargin = 56;
    $(".ddlabel").width(desiredWidth - dropDownMargin);

    /*
     * Adjust the drop-down menu's max height
     */
    var dropDownMenuChild = document.getElementById("msdropdown20_child");
    if(dropDownMenuChild)
    {
        var dropDownOffset = $(dropDownMenu).offset().top + $(dropDownMenu).height();
        var desiredMaxHeight = (dropDownMenuMaxHeight + dropDownOffset) > windowHeight ? 
                                windowHeight - dropDownOffset - kBottomSpace : dropDownMenuMaxHeight;
                               
        if(desiredMaxHeight != $(dropDownMenuChild).css("max-height"))
        {
            $(dropDownMenuChild).css("max-height", desiredMaxHeight);
        }
    }
    
    /*
     * Adjust the smart list edit pane width, and that of the contents
     */
     var smartList = document.getElementById("smartlist-pane");
     var smartListPaneWidth = fontListWidth;
     if ($(smartList).width() != smartListPaneWidth)
     {
         $(smartList).width(smartListPaneWidth);
     }

     var doneButton = document.getElementById("smartlist-done");
     var doneButtonWidth = doneButton.offsetWidth;
     var smartListNameInput = document.getElementById("smartlist-name-input");
     var inputWidth = smartListPaneWidth - doneButtonWidth - 18;    // padding
     if ($(smartListNameInput).width() != inputWidth)
     {
         $(smartListNameInput).width(inputWidth);
     }

     if(panel.isSmartListPaneVisible())
     {
        var doneButtonHeight = doneButton.offsetHeight;

        var fontCountDiv = document.getElementById("font-count");
        var fontCountHeight = fontCountDiv.offsetHeight;

        var smartListPane = document.getElementById("smartlist-pane");
        var smartListPaneTop = smartListPane.getBoundingClientRect().top;

        var availableHeight = windowHeight - kSearchBarContainerHeight - kToolbarContainerHeight - kBottomSpace;
        var treeHeight = (availableHeight/2) - (smartListPaneTop + doneButtonHeight) - fontCountHeight;

        // set the height of the ul fancytree-container class instead of the div.
        if($(".fancytree-container").height() != treeHeight)
        {
            $(".fancytree-container").height(treeHeight);
        }
     }
     
     /*
      * Adjust the quick search box size
      */
     var search = document.getElementById("search-bar");
     var paddingWidth = 33;
     if ($(search).width() != fontListWidth - paddingWidth)
     {
         $(search).width(fontListWidth - paddingWidth);
     }

     /*
      * Adjust font list size after we've tweaked all of the other main screen elements
      */
     var top = fontList.getBoundingClientRect().top;
     var height = (windowHeight - kSearchBarContainerHeight - kToolbarContainerHeight - kBottomSpace - top) + "px";
     if (fontList.style.height != height)
     {
         fontList.style.height = height;
     }

     screens = ["disabled-screen",
                "initial-loading-screen",
                "connection-failed-screen",
                "alert-screen",
                "about-screen",
                "utc-nologin-screen"];
     
     for (var screenNum = 0; screenNum < screens.length; ++screenNum)
     {
         var screen = document.getElementById(screens[screenNum]);
         
         if (screen.style.height != windowHeightPx)
         {
             screen.style.height = windowHeightPx;
         }
         
         if (screen.style.width != windowWidthPx)
         {
             screen.style.width = windowWidthPx;
         }
     }
     

    /*
     * Adjust the progress screen to the match the window
     */
    var progressScreen = document.getElementById("progress-screen");
    
    if (progressScreen.style.height != fontList.style.height)
    {
        progressScreen.style.height = fontList.style.height;
    }
    
    if ($(progressScreen).width() != fontListWidth)
    {
        $(progressScreen).width(fontListWidth);
    }
}

window.onresize = function(event)
{
    resizeElements();
    setScrollState();
};

function loadJSXCallback(object)
{
    console.log(object);
};

/**
 * Load JSX file into the scripting context of the product. 
 */
function loadJSX(fileName) 
{
    var csInterface = new CSInterface();
    var extensionRoot = csInterface.getSystemPath(SystemPath.EXTENSION) + "/jsx/";
    csInterface.evalScript('$.evalFile("' + extensionRoot + fileName + '")', loadJSXCallback);
}


/**
 * Update the theme with the AppSkinInfo retrieved from the host product.
 */
function updateThemeWithAppSkinInfo(appSkinInfo)
{

    // Update the background color of the panel elements
    var panelBackgroundColor = appSkinInfo.panelBackgroundColor.color;
    document.body.bgColor = toHex(panelBackgroundColor);
    document.getElementById("alert-screen").style.backgroundColor = document.body.bgColor;
    document.getElementById("progress-screen").style.backgroundColor = document.body.bgColor;
    
    var styleId = "ppstyle";

    var csInterface = new CSInterface();
    var appName = csInterface.hostEnvironment.appName;

    if (appName == "PHXS")
    {
        addRule(styleId,
                "button, select, input[type=button], input[type=submit]",
                "border-radius:3px;");
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////
    // NOTE: Below theme related code are only suitable for Photoshop. //
    // If you want to achieve same effect on other products please make your
    // own changes here. //
    // //////////////////////////////////////////////////////////////////////////////////////////////
    
    // Extensis notes: This code is (mostly) from Adobe with some fixes, but we are ignoring the above
    // comment and are also using it for InDesign and Illustrator. It seems to work fine.

    var gradientBg = "background-image: -webkit-linear-gradient(top, "
            + toHex(panelBackgroundColor, 40) + " , "
            + toHex(panelBackgroundColor, 10) + ");";
    var gradientDisabledBg = "background-image: -webkit-linear-gradient(top, "
            + toHex(panelBackgroundColor, 15)
            + " , "
            + toHex(panelBackgroundColor, 5) + ");";
    var boxShadow = "-webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.4), 0 1px 1px rgba(0, 0, 0, 0.2);";
    var boxActiveShadow = "-webkit-box-shadow: inset 0 1px 4px rgba(0, 0, 0, 0.6);";

    var isPanelThemeLight = panelBackgroundColor.red > 127;
    var fontColor, disabledFontColor;
    var borderColor;
    var inputBackgroundColor;
    var gradientHighlightBg;
    if (isPanelThemeLight)
    {
        fontColor = "#000000";
        disabledFontColor = "color:" + toHex(panelBackgroundColor, -70)
                + ";";
        borderColor = "border-color: " + toHex(panelBackgroundColor, -90)
                + ";";
        inputBackgroundColor = toHex(panelBackgroundColor, 54) + ";";
        gradientHighlightBg = "background-image: -webkit-linear-gradient(top, "
                + toHex(panelBackgroundColor, -40)
                + " , "
                + toHex(panelBackgroundColor, -50) + ");";
    }
    else
    {
        fontColor = "#ffffff";
        disabledFontColor = "color:" + toHex(panelBackgroundColor, 100)
                + ";";
        borderColor = "border-color: " + toHex(panelBackgroundColor, -45)
                + ";";
        inputBackgroundColor = toHex(panelBackgroundColor, -20) + ";";
        gradientHighlightBg = "background-image: -webkit-linear-gradient(top, "
                + toHex(panelBackgroundColor, -20)
                + " , "
                + toHex(panelBackgroundColor, -30) + ");";
    }

    // Update the default text style with pp values
    addRule(styleId, ".default", "font-size:" + appSkinInfo.baseFontSize
            + "px" + "; color:" + fontColor + "; background-color:"
            + toHex(panelBackgroundColor) + ";");
    addRule(
            styleId,
            "button, select, input[type=text], input[type=button], input[type=submit]",
            borderColor);
    addRule(styleId,
            "button, select, input[type=button], input[type=submit]",
            gradientBg);
    addRule(styleId,
            "button, select, input[type=button], input[type=submit]",
            boxShadow);
    addRule(
            styleId,
            "button:enabled:active, input[type=button]:enabled:active, input[type=submit]:enabled:active",
            gradientHighlightBg);
    addRule(
            styleId,
            "button:enabled:active, input[type=button]:enabled:active, input[type=submit]:enabled:active",
            boxActiveShadow);
    addRule(styleId, "[disabled]", gradientDisabledBg);
    addRule(styleId, "[disabled]", disabledFontColor);
    addRule(styleId, "input[type=text]", "padding:1px 3px;");
    addRule(styleId, "input[type=text]", "background-color: "
            + inputBackgroundColor)
            + ";";
    addRule(styleId, "input[type=text]:focus", "background-color: #ffffff;");
    addRule(styleId, "input[type=text]:focus", "color: #000000;");

	// Set any items with the defaultTextColor class to use our calculated font color
	// from above.  The above code is mostly from Adobe, this code is Extensis-born
    addRule(
            styleId,
            ".defaultTextColor",
            "color:" + fontColor + ";");
}

function addRule(stylesheetId, selector, rule)
{
    var stylesheet = document.getElementById(stylesheetId);

    if (stylesheet)
    {
        stylesheet = stylesheet.sheet;
        if (stylesheet.addRule)
        {
            stylesheet.addRule(selector, rule);
        }
        else if (stylesheet.insertRule)
        {
            stylesheet.insertRule(selector + ' { ' + rule + ' }',
                    stylesheet.cssRules.length);
        }
    }
}

function reverseColor(color, delta)
{
    return toHex(
    {
        red : Math.abs(255 - color.red),
        green : Math.abs(255 - color.green),
        blue : Math.abs(255 - color.blue)
    }, delta);
}

/**
 * Convert the Color object to string in hexadecimal format;
 */
function toHex(color, delta)
{
    function computeValue(value, delta)
    {
        var computedValue = !isNaN(delta) ? value + delta : value;
        computedValue = Math.floor(computedValue);
        if (computedValue < 0)
        {
            computedValue = 0;
        }
        else if (computedValue > 255)
        {
            computedValue = 255;
        }

        computedValue = computedValue.toString(16);
        return computedValue.length == 1 ? "0" + computedValue : computedValue;
    }

    var hex = "";
    if (color)
    {
        with (color)
        {
            hex = computeValue(red, delta) + computeValue(green, delta)
                    + computeValue(blue, delta);
        }
        ;
    }
    return "#" + hex;
}

function onAppThemeColorChanged(event)
{
    // Should get a latest HostEnvironment object from application.
    var skinInfo = new CSInterface().getHostEnvironment().appSkinInfo;
    // Gets the style information such as color info from the skinInfo,
    // and redraw all UI controls of your extension according to the style info.
    updateThemeWithAppSkinInfo(skinInfo);
}

