#include "Utilities.jsx"
#include "json2.js"

function applyFont(fontObject)
{
    if(app.documents.length > 0 && app.activeDocument.activeLayer)
    {
        if (app.activeDocument.activeLayer.locked)
        {
            alert(fontObject.singleLayerLockedErrorString);
            return false;
        }
        else 
        {
            var font = null;
            for (var i = 0; i < app.textFonts.length; ++i)
            {
                var textFont = app.textFonts[i];
                if (textFont != null)
                {
                    if (postScriptNamesMatch(textFont.name, fontObject.fontName))
                    {
                        font = textFont;
                        break;
                    }
                }
            }

            if(font != null)
            {
                setFontOnSelectedCharacterAttributes(font)
            }	
            return true;
        }
    }
			
    return false;
}

function setFontOnSelectedCharacterAttributes(font)
{
    if(app.documents.length > 0 && app.activeDocument.activeLayer && app.activeDocument.activeLayer.textFrames)
    {
        // If actual text is selected or is being edited, this is your man
        var selectionTextRange = app.activeDocument.selection;
        
        if(selectionTextRange.typename == "TextRange")
        {
            selectionTextRange.characterAttributes.textFont = font;
        }
        else if(app.activeDocument.selection.length > 0)
        {
            // Otherwise, if a text layer is selected this is what you'll get
            var selectedTextItems = app.activeDocument.selection;
            for(var i = 0; i < selectedTextItems.length; ++i)
            {
                var textItem = selectedTextItems[i];
                if (textItem.typename == "TextFrame")
                {
                    textItem.textRange.characterAttributes.textFont = font;
                }
                else if (textItem.typename == "GroupItem")
                {
                    setFontOnGroupItem(font, textItem);
                }
            }
        }
        else
        {
           app.activeDocument.characterStyles[0].characterAttributes.textFont = font;
        }
    }
}

function setFontOnGroupItem(font, groupItem)
{
    for(var i = 0; i < groupItem.textFrames.length; ++i)
    {
        groupItem.textFrames[i].textRange.characterAttributes.textFont = font;
    }

    for(var i = 0; i < groupItem.groupItems.length; ++i)
    {
        setFontOnGroupItem(font, groupItem.groupItems[i]);
    }
}

function isFontActivated(font)
{
    var returnObject = null;
    try
    {
        var illustratorFont = app.textFonts.getByName(font.name);
        returnObject = {isActivated: true, fontID: font.id};
    }
    catch(e)
    {
        returnObject = {isActivated: false, fontID: font.id};
    }
    return JSON.stringify(returnObject);
}