function FontList()
{
    this.fontRows = [];
    this.rowsByFontID = [];
    this.rowsByPostscriptName = {};
    this.cachedFontImages = [];
    
    this.didScroll             = false;
    this.tempPreserveSelection = false;
    this.activatedFontsByDisplayName = []; // an array of postscriptNames
    this.activatedFontsByPostscriptName = []; // an array of fontIDs
    
    this.applyTimer = null;
    
    this.fontListHandler = function(filename)
    {
        panel.fontListFile = filename;
    
        var toReplace = panel.isWindows() ? "file:///" : "file://";
        filename = filename.replace(toReplace, "");
    
        var result = cep.fs.readFile(filename);
        if (result.err == cep.fs.NO_ERROR)
        {
            this.loadFontList(result.data);
        }
    };

    this.fontPNGHandler = function(fonts)
    {
        for (var fontNum = 0; fontNum < fonts.length; ++fontNum)
        {
            var filename = fonts[fontNum][MessageKey.k2xPNGFilenameKey];
            
            if (filename == null || filename == "file://")
            {
                filename = fonts[fontNum][MessageKey.kPNGFilenameKey];
            }
            
            panel.requestedFontIDs[fonts[fontNum].fontID] = null;
            this.rowsByFontID[fonts[fontNum].fontID].setImageURL(filename);
        }
    };

    this.reload = function()
    {
        panel.messageSender.getSmartListFonts(panel.dropDownMenu.savedSmartListID);
    };
    
    this.reloadPreservingSelection = function()
    {
        this.tempPreserveSelection = true;
        panel.messageSender.getSmartListFonts(panel.dropDownMenu.savedSmartListID);
    };
    
    this.loadFontList = function(textFile)
    {
        var fontList = document.getElementById("font-list");
        
        var selectedPostscriptName = null;
        var scrollPosition = 0;
        if (this.tempPreserveSelection)
        {
            this.tempPreserveSelection = false;
            scrollPosition = fontList.scrollTop;
            
            if (panel.selectedRow != null)
            {
                selectedPostscriptName = panel.selectedRow.getPostscriptName();
            }
        }
        
        var lines = textFile.split("\n");

        this.rowsByFontID = [];
        this.rowsByPostscriptName = {};

        this.clearFontList();

        for (var lineNum = 0; lineNum < lines.length; ++lineNum)
        {
            var line = lines[lineNum];

            if (line != "")
            {
                var parts = line.split(/\t/);
                var fontID = parseInt(parts[0]);
                var displayName = parts[1];
                var postscriptName = parts[2];
                var activated = parseInt(parts[3]);
                var fontLocation = parts[4];
                var isLocal = parseInt(parts[5]);

                var row = new FontRow();

                row.setFontID(fontID);
                row.setDisplayName(displayName);
                row.setPostscriptName(postscriptName);
                row.setIsLocal(isLocal != 0);
                row.setActive(activated != ActivationMode.kInactive && activated != ActivationMode.kMissing);
                row.setFontLocation(fontLocation);
                row.setMissing(activated == ActivationMode.kMissing);

                this.fontRows[lineNum] = row;
            }
        }

        panel.messageSender.deleteTempFile(panel.fontListFile);

        for (var rowNum = 0; rowNum < this.fontRows.length; ++rowNum)
        {
            var currentRow = this.fontRows[rowNum];
            this.rowsByFontID[currentRow.fontID] = currentRow;

            // Duplicate postscript names are possible, keep the activated one or 
            // the first
            if (this.rowsByPostscriptName[currentRow.postscriptName] == null
                || currentRow.isActive())
            {
                this.rowsByPostscriptName[currentRow.postscriptName] = currentRow;
            }

            // The imageDiv needs to be a part of the font list for the mouse events to get 
            // hooked up properly when we set the image
            fontList.appendChild(currentRow.imageDiv);

            currentRow.setImage(this.cachedFontImages[currentRow.fontID]);
        }
        
        this.filteredRows = this.fontRows;

        this.restoreScrollPosition(scrollPosition, selectedPostscriptName);
        
        this.calculateVisibleRows(true);
        
        panel.updateFontCount();
    };

    this.restoreScrollPosition = function (scrollPosition, selectedPostscriptName)
    {
        var fontList = document.getElementById("font-list");
        
        fontList.scrollTop = scrollPosition;

        if (selectedPostscriptName == null)
        {
            new CSInterface().evalScript('updateSelection()', function(psName)
            {
                panel.fontList.selectRowByPostscriptName(psName);
            });
        }
        else
        {
            this.selectRowByPostscriptName(selectedPostscriptName);
        }
    };

    this.filter = function(text)
    {
        var fontList = document.getElementById("font-list");
        if(fontList)
        {
            if (text != null && this.fontRows != null)
            {
                this.searchText = text.toLocaleLowerCase();
                this.filteredRows = [];
                for(var i = 0; i < this.fontRows.length; ++i)
                {
                    var row = this.fontRows[i];
                    if (row != null && row.matchesSearch(this.searchText))
                    {
                        this.filteredRows.push(row);
                    }
                }
            }
            else
            {
                this.searchText = null;
                this.filteredRows = this.fontRows;
            }
            $(fontList).empty();
            for (var rowNum = 0; rowNum < this.filteredRows.length; ++rowNum)
            {
                var currentRow = this.filteredRows[rowNum];
    
                // The imageDiv needs to be a part of the font list for the mouse events to get 
                // hooked up properly when we set the image
                fontList.appendChild(currentRow.imageDiv);
    
                currentRow.installTooltips();
                currentRow.setImage(this.cachedFontImages[currentRow.fontID]);
            }
    
            this.restoreScrollPosition(0, null);
            
            this.calculateVisibleRows(true);
        }
    };

    this.clearSearch = function()
    {
        var search = document.getElementById("search-input");
        search.value = "";

        this.filter(search.value);
    };

    this.clearFontList = function()
    {
        // clears child elements, including jQuery data
        $('#font-list').empty();

        this.fontRows = [];
        this.filteredRows = [];
        this.rowsByFontID = [];
        this.rowsByPostscriptName = {};
    };
    

    this.selectRowByPostscriptName = function(psName)
    {
        if (psName != null)
        {
            if (panel.selectedRow == null || panel.selectedRow.postscriptName != psName )
            {
                var row = this.rowsByPostscriptName[psName];

                if (row != null)
                {
                    panel.setSelectedRow(row);
                    this.scrollToRow(row);
                }
            }
        }
    };
    
    this.selectNext = function()
    {
        var row = null;
        
        this.resetApplyTimer();
        
        if (panel.selectedRow != null)
        {
            row = panel.selectedRow;
            var rowIndex = this.fontRows.indexOf(panel.selectedRow);
            
            if (rowIndex < this.fontRows.length - 1)
            {
                rowIndex += 1;
                row = this.fontRows[rowIndex];
            }
        }
        else if (this.fontRows.length > 0)
        {
            row = this.fontRows[this.fontRows.length - 1];
        }
        
        if (row != null)
        {
            panel.setSelectedRow(row);
            this.scrollToRow(row);
        }
    };
    
    this.selectPrevious = function()
    {
        var row = null;
        
        this.resetApplyTimer();
        
        if (panel.selectedRow != null)
        {
            row = panel.selectedRow;
            var rowIndex = this.fontRows.indexOf(panel.selectedRow);
            
            if (rowIndex > 0)
            {
                rowIndex -= 1;
                row = this.fontRows[rowIndex];
            }
        }
        else if (this.fontRows.length > 0)
        {
            row = this.fontRows[0];
        }
        
        if (row != null)
        {
            panel.setSelectedRow(row);
            this.scrollToRow(row);
        }
    };
    
    this.clearApplyTimer = function()
    {
        if (this.applyTimer != null)
        {
            window.clearTimeout(this.applyTimer);
        }
        
        this.applyTimer = null;
    };
    
    this.resetApplyTimer = function()
    {
        this.clearApplyTimer();
        
        this.applyTimer = window.setTimeout(function()
        {
            if (panel.selectedRow != null)
            {
                panel.selectedRow.simulateClick();
                panel.fontList.clearApplyTimer();
            }
        }, 500);
    };
    
    this.isApplyingFont = function()
    {
        return this.applyTimer != null;
    };
    
    this.scrollToRow = function(rowToSelect)
    {
        var rowY      = -1;
        var foundRow  = false;
        
        for (var rowNum = 0; rowNum < this.filteredRows.length; ++rowNum)
        {
            var row = this.filteredRows[rowNum];
            
            if (row.fontID == rowToSelect.fontID)
            {
                foundRow = true;
                break;
            }
            
            rowY += fontRowHeight;
        }
        
        if (rowY < 0)
        {
            rowY = 0;
        }
        
        
        if (foundRow)
        {
            var fontList = document.getElementById("font-list");
            var scrollPosition = fontList.scrollTop;
            
            if (rowY < scrollPosition)
            {
                fontList.scrollTop = rowY + 1;
                this.calculateVisibleRows(true);
            }
            else if (rowY + fontRowHeight > scrollPosition + $(fontList).height())
            {
                fontList.scrollTop = rowY - $(fontList).height() + fontRowHeight;
                this.calculateVisibleRows(true);
            }
        }
    };

    this.calculateVisibleRows = function(forceLoadImages)
    {
        var fontList = document.getElementById("font-list");
        var scrollPosition = fontList.scrollTop;

        var numVisibleRows = Math.ceil(fontList.clientHeight / fontRowHeight);
        var firstRow = Math.floor(scrollPosition / fontRowHeight);
        var lastRow = firstRow + numVisibleRows;
        
        if (this.fontRows.indexOf(panel.selectedRow) < 0)
        {
            panel.setSelectedRow(null);
        }

        if(forceLoadImages || firstRow != panel.lastFirstRow || lastRow != panel.lastLastRow)
        {
            panel.lastFirstRow = firstRow;
            panel.lastLastRow = lastRow;

            var rowIndicies = {indexOfFirstRow: firstRow, indexOfLastRow: lastRow};
            this.loadImages(rowIndicies);
        }
    };

    this.loadImages = function(rowIndicies)
    {
        var allFontIDs = [];
        var highPriorityFontIDs = [];

        for (var rowIndex = rowIndicies.indexOfFirstRow; rowIndex < this.filteredRows.length && rowIndex <= rowIndicies.indexOfLastRow; ++rowIndex)
        {
            var row = this.filteredRows[rowIndex];

            if(row.getImage() == null)
            {
                var cachedFontPNG = this.cachedFontImages[row.fontID];

                if(cachedFontPNG == null)
                {
                    var alreadyRequested = panel.requestedFontIDs[row.fontID];
                    if(alreadyRequested == null)
                    {
                        allFontIDs.push(row.fontID);
                        highPriorityFontIDs.push(row.fontID);
                        panel.requestedFontIDs[row.fontID] = row.fontID;
                    }
                }
                else
                {
                    //  Cache hit: Load the PNG from the cache
                    row.setImage(cachedFontPNG);
                }                                
            }
        }

        if(allFontIDs.length > 0)
        {
            panel.messageSender.getFontPNGs(allFontIDs, highPriorityFontIDs, 0xffffffff);
        }
    };
    
    this.setActive = function(fontID, displayName, postscriptName)
    {
        this.activatedFontsByDisplayName[displayName] = postscriptName;
        this.activatedFontsByPostscriptName[postscriptName] = fontID;
    };
    
    this.setInactive = function(fontID, displayName, postscriptName)
    {
        this.activatedFontsByDisplayName[displayName] = null;
        this.activatedFontsByPostscriptName[postscriptName] = null;
    };
    
    this.canActivateFont = function(fontID, displayName, postscriptName)
    {
        if (this.activatedFontsByDisplayName[displayName] != null && 
            this.activatedFontsByDisplayName[displayName] != postscriptName)
        {
            return false;
        }
        else
        {
            if (this.activatedFontsByPostscriptName[postscriptName] != null && 
                this.activatedFontsByPostscriptName[postscriptName] != fontID)
            {
                return false;
            }
            else
            {
                return true;
            }
        }
    };
};

$(document).keydown(function(e)
{
    if ($("#font-list")[0] == document.activeElement)
    {
        if (e.which == Keyboard.DOWN_ARROW)
        {
            panel.fontList.selectNext();
        }
        else if (e.which == Keyboard.UP_ARROW)
        {
            panel.fontList.selectPrevious();
        }
        
        return false;
    }
    
    return true;
});

// --------------------------------------------------------------------
// Delay when we trigger off a scroll so we don't do it for every pixel

function setScrollState()
{
    panel.fontList.didScroll = true;
};

function filterFontList()
{
    var search = document.getElementById("search-input");
    var text = search.value;
    
    panel.fontList.filter(text);
};


setInterval(function()
    {
    if (panel.fontList.didScroll)
    {
        panel.fontList.didScroll = false;
        panel.fontList.calculateVisibleRows(false);
        
        var tooltipObject = panel.getTooltipObject();
        if(tooltipObject)
        {
            tooltipObject.removeChild(tooltipObject.tooltip);
            panel.setTooltipObject(null);
        }
    }
    }, 100);
//--------------------------------------------------------------------
