function ProgressInfo()
{
    this.message = null;
    this.step = null;
    this.numSteps = null;
    this.progressRunning = false;
};

function Progress()
{
    this.googleProgressInfo = new ProgressInfo();
    this.bulkReplicationProgressInfo = new ProgressInfo();
    this.incrementalReplicationProgressInfo = new ProgressInfo();
    this.fontServerReplicationProgressInfo = new ProgressInfo();
    this.indeterminateSpinner = new IndeterminateSpinner("progress-image");
    
    this.setProgressInfo = function(progressInfo, text, step, numSteps, progressRunning)
    {
        progressInfo.text = text;
        progressInfo.step = step;
        progressInfo.numSteps = numSteps;
        progressInfo.progressRunning = progressRunning;
    };
    
    /**
     * Update the progress image and text based on title, message, and the current progress state.
     * This handles both indeterminate and determinate progress, and will show the progress screen
     * if necessary
     */
    this.updateProgress = function(title, progressInfo)
    {
        var progressScreen     = document.getElementById("progress-screen");
        var progressImage      = document.getElementById("progress-image");
        var progressTitle      = document.getElementById("progress-title");
        var progressMessage    = document.getElementById("progress-message");
        var progressPercentage = document.getElementById("progress-percentage");
        

        if(progressTitle.innerHTML != title)
        {
            progressTitle.innerHTML = title;
        }

        if(progressMessage.innerHTML != progressInfo.message)
        {
            progressMessage.innerHTML = progressInfo.message;
        }
        
        var imagePath        = null;
        var percentageString = localizeString("PercentCompleteFormat");

        if(progressInfo.step < progressInfo.numSteps)
        {
            this.stopIndeterminateProgressSpinner();
            var fraction   = 1.0;
            var percentage = 100;

            if(progressInfo.step >=0 && progressInfo.numSteps > 0)
            {
                fraction = progressInfo.step/progressInfo.numSteps;
                percentage = Math.floor(fraction * 100);
            }

            percentageString = percentageString.replace("{0}", percentage.toString());

            // pick one of the 12 slices to show
            imagePath = "img/progress" + Math.floor((percentage*12.0)/100.0) + ".png";

            if(progressImage.src != imagePath)
            {
                progressImage.src = imagePath;
            }
        }
        else
        {
            percentageString = "";
            
            this.startIndeterminateProgressSpinner();
        }


        if(progressPercentage.innerHTML != percentageString)
        {
            progressPercentage.innerHTML = percentageString;
        }
        
        if(!progressScreen.classList.contains("progress-screen"))
        {            
            progressScreen.className = "progress-screen";
        }
    };
    
    /**
     * Hide the progress screen
     */
    this.hideProgress = function()
    {
        this.stopIndeterminateProgressSpinner();

        var progressScreen = document.getElementById("progress-screen");
        if(progressScreen.classList.contains("progress-screen"))
        {            
            progressScreen.className = "progress-screen-hidden";
        }
    };

    this.startIndeterminateProgressSpinner = function()
    {
        this.indeterminateSpinner.startSpinner();
    };
        
    this.stopIndeterminateProgressSpinner = function()
    {
        this.indeterminateSpinner.stopSpinner();
    };
};